// This code will output timestamps.  This data will be output to the serial port as well as an SD card.
// This uses and Adafruit Data Logging Shield
// By: Spencer McDonald
// 2/19/14

// To connect the SD/RTC shield, plug it in.

// The RTC uses the I2C bus [A4 and A5]. The SD card uses digital pins 10-13.

// Libraries to include

#include "SD.h"                    // SD library

#include <Wire.h>                  // RTC library
#include "RTClib.h"

// Initalize Variables

#define greenLEDpin 4

const int chipSelect = 10; // Adafruit uses 10, Sparkfun uses 8
                                                  
File datalog;
char filename[] = "LOGGER00.csv";
RTC_DS1307 RTC;

void setup()
{
  Serial.begin(115200); // Opens the communication between Arduino and computer serial port
  
  Wire.begin();         // Begins RTC communication
  RTC.begin();
  
// Check if RTC is running, if not fix it

  if (! RTC.isrunning()) {
    Serial.println("RTC is NOT running!");
    // following line sets the RTC to the date & time this sketch was compiled
    RTC.adjust(DateTime(__DATE__, __TIME__));
  }


// Initalize SD Card  
  
  Serial.print("Intializing SD Card...");
  pinMode(10, OUTPUT); // Necessary for microSD shield
  
  if(!SD.begin(chipSelect)) 
  {
    Serial.println("Card failed, or not present");
    return;
  }
  Serial.println("Card initialized.");  
  
  Serial.print("Creating File...");
  

// Make a new file each time the arduino is powered

  for (uint8_t i = 0; i < 100; i++) 
  {
    filename[6] = i/10 + '0';
    filename[7] = i%10 + '0';
    if (! SD.exists(filename)) 
    {
      // only open a new file if it doesn't exist
      datalog = SD.open(filename, FILE_WRITE); 
      break;
    }
  }
  
  Serial.print("Logging to: ");
  Serial.println(filename);
  
  if(!datalog)
  {
  Serial.println("Couldn't Create File"); 
  return;
  }
  
// Print Header
  
  String Header = "RTC Date, RTC Time";   
  
  datalog = SD.open(filename, FILE_WRITE);
  datalog.println(Header);
  Serial.println(Header); 
   
  datalog.close();
  
  digitalWrite(greenLEDpin, OUTPUT);
}

//___________________________________________________________________________________

void loop()
{  
datalog = SD.open(filename, FILE_WRITE);
 
// Real Time Clock

  DateTime now = RTC.now();

  datalog.print(now.month(), DEC);
  datalog.print("/");
  datalog.print(now.day(), DEC);
  datalog.print("/");
  datalog.print(now.year(), DEC);
  datalog.print(",");
  datalog.print(now.hour(), DEC);
  datalog.print(":");
  datalog.print(now.minute(), DEC);
  datalog.print(":");
  datalog.print(now.second(), DEC);
  datalog.print(",");
 
  Serial.print(now.month(), DEC);
  Serial.print("/");
  Serial.print(now.day(), DEC);
  Serial.print("/");
  Serial.print(now.year(), DEC);
  Serial.print(", ");
  Serial.print(now.hour(), DEC);
  Serial.print(":");
  Serial.print(now.minute(), DEC);
  Serial.print(":");
  Serial.print(now.second(), DEC);
  Serial.print(", ");
  
// Close the file and flash LED to show data was recorded

datalog.println("");
Serial.println("");

datalog.close(); 

digitalWrite(greenLEDpin, HIGH);  //Flash the LED to show it is working
delay(1000);
digitalWrite(greenLEDpin, LOW);
}



