// This code will output accelerations using a digital accelerometer (ADXL345 Breakout, Sparkfun: SEN-09836).
// By: Spencer McDonald

// To connect the accelerometer sensor: GND, Vcc = 3.3V, CS = 3.3V, INT1 = N/A, INT2 = N/A, SD0 = N/A, SDA = A4, SCL = A5.
// This code uses the I2C bus [A4 and A5].

// Libraries to include

#include <Wire.h>

// Initalize Variables

#define LEDpin 4

#define DEVICE (0x53) // Accelerometer device address as specified in data sheet 

byte _buff[6];

char POWER_CTL = 0x2D;	//Power Control Register
char DATA_FORMAT = 0x31;
char DATAX0 = 0x32;	//X-Axis Data 0
char DATAX1 = 0x33;	//X-Axis Data 1
char DATAY0 = 0x34;	//Y-Axis Data 0
char DATAY1 = 0x35;	//Y-Axis Data 1
char DATAZ0 = 0x36;	//Z-Axis Data 0
char DATAZ1 = 0x37;	//Z-Axis Data 1

float accelX = 0;
float accelY = 0;
float accelZ = 0;

void setup()
{
  Wire.begin();        // join i2c bus
  Serial.begin(115200); // Opens the communication between Arduino and computer serial port
   
//Put the ADXL345 into +/- 4G range by writing the value 0x01 to the DATA_FORMAT register.
  writeTo(DATA_FORMAT, 0x01);

//Put the ADXL345 into Measurement Mode by writing 0x08 to the POWER_CTL register.
  writeTo(POWER_CTL, 0x08);
  
  digitalWrite(LEDpin, OUTPUT); 
}


void loop()
{ 

// Digital Accelerometer

  uint8_t howManyBytesToRead = 6;
  
  readFrom( DATAX0, howManyBytesToRead, _buff); //read the acceleration data from the ADXL345

// each axis reading comes in 10 bit resolution, ie 2 bytes.  Least Significat Byte first!!
// thus we are converting both bytes in to one int

  int x = (((int)_buff[1]) << 8) | _buff[0];   
  int y = (((int)_buff[3]) << 8) | _buff[2];
  int z = (((int)_buff[5]) << 8) | _buff[4];
  
  accelX = (x*(5.0/1024)-0.08)/0.6;
  accelY = (y*(5.0/1024)-0.07)/0.6;
  accelZ = (z*(5.0/1024))/0.6;
  
  Serial.print("[X,Y,Z] = ");
  Serial.print(accelX);
  Serial.print(", ");
  Serial.print(accelY);
  Serial.print(", ");
  Serial.print(accelZ);
  Serial.println();
  
  digitalWrite(LEDpin,HIGH);                                     // Flash LED to show sensor running
  delay(500);                                                    // Print about once per second
  digitalWrite(LEDpin,LOW);

}

// Functions used in the loop function

void writeTo(byte address, byte val) {
  Wire.beginTransmission(DEVICE); // start transmission to device 
  Wire.write(address);             // send register address
  Wire.write(val);                 // send value to write
  Wire.endTransmission();         // end transmission
}


// Reads num bytes starting from address register on device in to _buff array
void readFrom(byte address, int num, byte _buff[]) {
  Wire.beginTransmission(DEVICE); // start transmission to device 
  Wire.write(address);             // sends address to read from
  Wire.endTransmission();         // end transmission

  Wire.beginTransmission(DEVICE); // start transmission to device
  Wire.requestFrom(DEVICE, num);    // request 6 bytes from device

  int i = 0;
  while(Wire.available())         // device may send less than requested (abnormal)
  { 
    _buff[i] = Wire.read();    // receive a byte
    i++;
  }
  Wire.endTransmission();         // end transmission
}
